unit BMVIEW;

interface

uses WinTypes, WinProcs, WinDos, OWindows, ODialogs, Strings;

{$R BMVIEW}

{ Include resource file constants }

{the following from oStdDlgs.inc}
const
  sd_FileOpen = $7FFF;          { Use default file open template }
  sd_FileSave = $7FFE;          { Use default file save template }
  sd_WNFileOpen = $7F00;        { Normal file open template }
  sd_WNFileSave = $7F01;        { Nomral file save template }
  sd_BCFileOpen = $7F03;        { BWCC file open template }
  sd_BCFileSave = $7F04;        { BWCC file save template }

const
  id_FName      = 100;
  id_FPath      = 101;
  id_FList      = 102;
  id_DList      = 103;
  id_BImage     = 110;
  id_CheckImage = 111;

{ end of ostddlgs.inc}

const
  fsFileSpec = fsFileName + fsExtension;

type
  PFileDialog = ^TFileDialog;
  TFileDialog = object(TDialog)
    Caption: PChar;
    FilePath: PChar;
    OldFileName: array[0..fsPathName] of Char;
    PathName: array[0..fsPathName] of Char;
    Extension: array[0..fsExtension] of Char;
    FileSpec: array[0..fsFileSpec] of Char;
    BitMapHandle: HBitmap;
    Width, Height: LongInt;
    Brush : HBRUSH;
    Shrink : boolean;
    constructor Init(AParent: PWindowsObject; AName, AFilePath: PChar);
    destructor Done; virtual;
    function CanClose: Boolean; virtual;
    procedure SetupWindow; virtual;
    procedure HandleFName(var Msg: TMessage); virtual id_First + id_FName;
    procedure GetBitmapData(var TheFile: File; BitsHandle: THandle; BitsByteSize: Longint);
    function OpenDIB(var TheFile: File): Boolean;
    function LoadBitmapFile(Name: PChar): Boolean;
    procedure PaintBitmap;
    procedure HandleCheckBox(var Msg: TMessage); virtual id_First +id_CheckImage;
    procedure HandleFList(var Msg: TMessage); virtual id_First + id_FList;
    procedure HandleDList(var Msg: TMessage); virtual id_First + id_DList;
  private
    procedure SelectFileName;
    procedure UpdateFileName;
    function UpdateListBoxes: Boolean;
  end;

const
  sd_WNInputDialog = $7F02;     { Normal input dialog template }
  sd_BCInputDialog = $7F05;     { BWCC input dialog template }

implementation

uses
      bwcc, omemory;

procedure AHIncr; far; external 'KERNEL' index 114;

function GetFileName(FilePath: PChar): PChar;
  var
       P: PChar;

  begin
    P := StrRScan(FilePath, '\');
    if P = nil then
      P := StrRScan(FilePath, ':');
    if P = nil then
      GetFileName := FilePath
    else
      GetFileName := P + 1
  end;

function GetExtension(FilePath: PChar): PChar;
  var
       P: PChar;

  begin
    P := StrScan(GetFileName(FilePath), '.');
    if P = nil then
      GetExtension := StrEnd(FilePath)
    else
      GetExtension := P
  end;

function HasWildCards(FilePath: PChar): Boolean;
begin
  HasWildCards := (StrScan(FilePath, '*') <> nil) or (StrScan(FilePath, '?') <> nil)
end;

{ TFileDialog }

constructor TFileDialog.Init(AParent: PWindowsObject; AName, AFilePath: PChar);
  begin
    if AName = PChar(sd_FileOpen) then
      AName := PChar(sd_BCFileOpen);
    if AName = PChar(sd_FileSave) then
      AName := PChar(sd_BCFileSave);
    TDialog.Init(AParent, AName);
    Caption := nil;
    FilePath := AFilePath;
    fillchar(OldFileName, sizeof(OldFileName), 0);
    BitMapHandle := 0;
    Brush := CreateSolidBrush($C0C0C0);
    Shrink := TRUE
  end;

destructor TFileDialog.Done;
  begin
    if BitMapHandle <> 0 then
      DeleteObject(BitMapHandle);
    DeleteObject(Brush);
    inherited done
  end;

function TFileDialog.CanClose: Boolean;
  var
       PathLen: Word;

  begin
    CanClose := False;
    GetDlgItemText(HWindow, id_FName, PathName, fsPathName + 1);
    FileExpand(PathName, PathName);
    PathLen := StrLen(PathName);
    if (PathName[PathLen - 1] = '\') or HasWildCards(PathName) or (GetFocus = GetDlgItem(HWindow, id_DList)) then
    begin
      if PathName[PathLen - 1] = '\' then
        StrLCat(PathName, FileSpec, fsPathName);
      if not UpdateListBoxes then
      begin
        MessageBeep(0);
        SelectFileName;
      end;
      Exit
    end;
    StrLCat(StrLCat(PathName, '\', fsPathName), FileSpec, fsPathName);
    if UpdateListBoxes then
      Exit;
    PathName[PathLen] := #0;
    if GetExtension(PathName)[0] = #0 then
      StrLCat(PathName, Extension, fsPathName);
    AnsiLower(StrCopy(FilePath, PathName));
    CanClose := True
  end;

procedure TFileDialog.SetupWindow;
  begin
    SendDlgItemMessage(HWindow, id_FName, em_LimitText, fsPathName, 0);
    if Caption <> nil then
      SetWindowText(HWindow, Caption);
    StrLCopy(PathName, FilePath, fsPathName);
    StrLCopy(Extension, GetExtension(PathName), fsExtension);
    if HasWildCards(Extension) then
      Extension[0] := #0;
    if not UpdateListBoxes then
    begin
      StrCopy(PathName, '*.*');
      UpdateListBoxes
    end;
    SelectFileName;
    SendMessage(GetDlgItem(hWindow, id_CheckImage), bm_SetCheck, bf_Checked, 0)
  end;

procedure TFileDialog.HandleFName(var Msg: TMessage);
  begin
    if Msg.LParamHi = en_Change then
      EnableWindow(GetDlgItem(HWindow, id_Ok), SendMessage(Msg.LParamLo, wm_GetTextLength, 0, 0) <> 0)
  end;

procedure TFileDialog.GetBitmapData(var TheFile: File; BitsHandle: THandle; BitsByteSize: Longint);
  type
        LongType = record
                     case Word of
                       0: (Ptr: Pointer);
                       1: (Long: Longint);
                       2: (Lo: Word;
	                   Hi: Word);
                   end;

  var
       Count : Longint;
       Start, ToAddr, Bits: LongType;

  begin
    Start.Long := 0;
    Bits.Ptr := GlobalLock(BitsHandle);
    Count := BitsByteSize - Start.Long;
    while Count > 0 do
    begin
      ToAddr.Hi := Bits.Hi + (Start.Hi * Ofs(AHIncr));
      ToAddr.Lo := Start.Lo;
      if Count > $4000 then
        Count := $4000;
      BlockRead(TheFile, ToAddr.Ptr^, Count);
      Start.Long := Start.Long + Count;
      Count := BitsByteSize - Start.Long;
    end;
    GlobalUnlock(BitsHandle)
  end;

function TFileDialog.OpenDIB(var TheFile: File): Boolean;
  var
     bitCount        : Word;
     size            : Word;
     longWidth       : Longint;
     DCHandle        : HDC;
     BitsPtr         : Pointer;
     BitmapInfo      : PBitmapInfo;
     BitsHandle      : THandle;
     NewBitmapHandle : THandle;
     NewPixelWidth   : word;
     NewPixelHeight  : Word;

  begin
    OpenDIB := True;
    Seek(TheFile, 28);
    BlockRead(TheFile, bitCount, SizeOf(bitCount));
    if bitCount <= 8 then
      begin
        size := SizeOf(TBitmapInfoHeader) + ((1 shl bitCount) * SizeOf(TRGBQuad));
        BitmapInfo := MemAlloc(size);
        Seek(TheFile, SizeOf(TBitmapFileHeader));
        BlockRead(TheFile, BitmapInfo^, size);
        NewPixelWidth := BitmapInfo^.bmiHeader.biWidth;
        NewPixelHeight := BitmapInfo^.bmiHeader.biHeight;
        longWidth := (((NewPixelWidth * bitCount) + 31) div 32) * 4;
        BitmapInfo^.bmiHeader.biSizeImage := longWidth * NewPixelHeight;
        {GlobalCompact(-1);}
        BitsHandle := GlobalAlloc(gmem_Moveable or gmem_Zeroinit, BitmapInfo^.bmiHeader.biSizeImage);
        GetBitmapData(TheFile, BitsHandle, BitmapInfo^.bmiHeader.biSizeImage);
        DCHandle := CreateDC('Display', nil, nil, nil);
        BitsPtr := GlobalLock(BitsHandle);
        NewBitmapHandle := CreateDIBitmap(DCHandle, BitmapInfo^.bmiHeader, cbm_Init, BitsPtr, BitmapInfo^, 0);
        DeleteDC(DCHandle);
        GlobalUnlock(BitsHandle);
        GlobalFree(BitsHandle);
        FreeMem(BitmapInfo, size);
        if NewBitmapHandle <> 0 then
          begin
            if BitmapHandle <> 0 then DeleteObject(BitmapHandle);
            BitmapHandle := NewBitmapHandle;
            Width := NewPixelWidth;
            Height := NewPixelHeight;
          end
        else
          OpenDIB := False;
      end
    else
      OpenDIB := False
  end;

function TFileDialog.LoadBitmapFile(Name: PChar): Boolean;
  var
       TheFile         : File;
       TestWin30Bitmap : Longint;

  begin
    LoadBitmapFile := False;
    Assign(TheFile, Name);
    Reset(TheFile, 1);
    Seek(TheFile, 14);
    BlockRead(TheFile, TestWin30Bitmap, SizeOf(TestWin30Bitmap));
    if TestWin30Bitmap = 40 then
      if OpenDIB(TheFile) then
        LoadBitmapFile := True
      else
        BWCCMessageBox(HWindow, 'Unable to create Windows 3.x bitmap from file.', Name, mb_Ok)
    else
      BWCCMessageBox(HWindow, 'Not a Windows 3.x bitmap file.  Convert using Paintbrush.', Name, mb_Ok);
    Close(TheFile)
  end;

procedure TFileDialog.PaintBitmap;
  var
       MemDC     : HDC;
       OldBitmap : HBitmap;
       R         : TRect;
       dc        : HDC;
       Points    : ^TPoint;
       window    : HWND;
       Rgn       : HRGN;
       x,y       : integer;

  begin
    if BitMapHandle <> 0 then
    begin
      window := GetDlgItem(hWindow, id_Bimage);
      dc := GetDC(window);
      GetClientRect(window,R);
      FillRect(dc, R, Brush);
      with R do
        Rgn := CreateRectRgn(Left, Top, Right, Bottom);
      SelectClipRgn(dc, Rgn);
      MemDC := CreateCompatibleDC(dc);
      SelectObject(MemDC, BitMapHandle);
      GetWindowRect(window, R);
      Points := @R;
      ScreenToClient(window, Points^);
      Points := @R.Right;
      ScreenToClient(window, Points^);
      SetCursor(LoadCursor(0, idc_Wait));
      x := 0; y := 0;
      if R.Right > Width then
        x := (R.Right -Width) div 2;
      if R.Bottom > Height then
        y := (R.Bottom -Height) div 2;
      if Shrink then
        begin
          if (R.Right > Width) or (R.Bottom > Height) then
            StretchBlt(dc, x, y, Width, Height, MemDC, 0, 0, Width, Height, SRCCopy)
          else
            StretchBlt(dc, x, y, R.Right, R.Bottom, MemDC, 0, 0, Width, Height, SRCCopy)
        end
      else
        StretchBlt(dc, x, y, Width, Height, MemDC, 0, 0, Width, Height, SRCCopy);
      SetCursor(LoadCursor(0, idc_Arrow));
      DeleteDC(MemDC);
      DeleteObject(Rgn);
      ReleaseDC(window, dc)
    end
  end;

procedure TFileDialog.HandleCheckBox(var Msg: TMessage);
  begin
    Shrink := not Shrink;
    PaintBitmap
  end;

procedure TFileDialog.HandleFList(var Msg: TMessage);
  var
       LoadFileName : array[0..fsPathName] of Char;

  begin
    case Msg.LParamHi of
      lbn_SelChange,
      lbn_DblClk     : begin
                         DlgDirSelect(HWindow, PathName, id_FList);
                         FileExpand(LoadFileName, PathName);
                         if StrComp(OldFileName, LoadFileName) <> 0 then
                           if LoadBitmapFile(LoadFileName) then
                           begin
                             StrCopy(OldFileName, LoadFileName);
                             PaintBitmap
                           end;
                         UpdateFileName;
                         if Msg.LParamHi = lbn_DblClk then
                           Ok(Msg)
                       end;
      lbn_KillFocus  : SendMessage(Msg.LParamLo, lb_SetCurSel, Word(-1), 0)
    end
  end;

procedure TFileDialog.HandleDList(var Msg: TMessage);
  begin
    case Msg.LParamHi of
      lbn_SelChange,
      lbn_DblClk     : begin
                         DlgDirSelect(HWindow, PathName, id_DList);
                         StrCat(PathName, FileSpec);
                         if Msg.LParamHi = lbn_DblClk then
                           UpdateListBoxes else
                         UpdateFileName
                       end;
      lbn_KillFocus  : SendMessage(Msg.LParamLo, lb_SetCurSel, Word(-1), 0)
    end
  end;

procedure TFileDialog.SelectFileName;
  begin
    SendDlgItemMessage(HWindow, id_FName, em_SetSel, 0, 0);
    SetFocus(GetDlgItem(HWindow, id_FName))
  end;

procedure TFileDialog.UpdateFileName;
  begin
    SetDlgItemText(HWindow, id_FName, AnsiLower(PathName));
    SendDlgItemMessage(HWindow, id_FName, em_SetSel, 0, 0)
  end;

function TFileDialog.UpdateListBoxes: Boolean;
  var
       Result : Integer;
       Path   : array[0..fsPathName] of Char;

  begin
    UpdateListBoxes := False;
    if GetDlgItem(HWindow, id_FList) <> 0 then
      begin
        StrCopy(Path, PathName);
        Result := DlgDirList(HWindow, Path, id_FList, id_FPath, 0);
        if Result <> 0 then
          DlgDirList(HWindow, '*.*', id_DList, 0, $C010)
      end
    else
      begin
        StrLCopy(Path, PathName, GetFileName(PathName) - PathName);
        StrLCat(Path, '*.*', fsPathName);
        Result := DlgDirList(HWindow, Path, id_DList, id_FPath, $C010);
      end;
    if Result <> 0 then
    begin
      StrLCopy(FileSpec, GetFileName(PathName), fsFileSpec);
      StrCopy(PathName, FileSpec);
      UpdateFileName;
      UpdateListBoxes := True
    end
  end;


end.
