(***************************************************************************
  SerTest
  Serial test program
  PJB December 31, 1993, CompuServe mail to INTERNET:d91-pbr@nada.kth.se
  Copyright PJB 1993, All Rights Reserved.
  Free source, use at your own risk.
  If modified, please state so if you pass this around.

  Very very silly, very slow terminal program to demonstrate serial code.
  Lets me log on to my campus UNIX host

  TTerminal is a sloooow device...

***************************************************************************)
program SerTest;

  uses
    Dos,
    App, Dialogs, Drivers, Menus, MsgBox, Objects, TextView, Validate, Views,
    toyPrefs, HelpCtx,
    COMSlect, Serial, toyApp, toyUtils, TVUtils;

  type
    PMyTerminal = ^TMyTerminal;
    TMyTerminal=
      object (TTerminal)
        COMPort : PSerialStream;
        procedure HandleEvent(var Event:TEvent); virtual;
      end;

    TSerTestApp =
      object (TToyApp)
        COMPort : TSerialStream;
        Term    : PMyTerminal;
        constructor Init;
        procedure HandleEvent(var Event:TEvent); virtual;
        procedure Idle; virtual;
        procedure InitCOM;
        procedure InitMenubar; virtual;
        procedure Terminal;
      end;


  const
    TermBufSize      = 8192;

    toyStart         = 100;
    cmCOMSelect      = toyStart+0;
    cmHangup         = toyStart+1;
    cmDial           = toyStart+2;
    cmCOMParam       = toyStart+3;

  var
    TermText : Text;


(***************************************************************************
***************************************************************************)

  procedure TMyTerminal.HandleEvent;
  begin
    inherited HandleEvent(Event);
    if (Event.What=evKeyDown) and (Event.CharCode<>#0) then
      COMPort^.Write(Event.CharCode, 1);
  end;


(***************************************************************************
***************************************************************************)


  (*******************************************************************
    Init
  *******************************************************************)
  constructor TSerTestApp.Init;
  begin
    inherited Init;

    (* Show some text *)
    HelpFileName:='HELPTEST.HLP';
    ShowHelp(hcSerialIntro);

    InitCOM;
  end;


  (*******************************************************************
    Check for incoming characters
  *******************************************************************)
  procedure TSerTestApp.Idle;
    type
      PTextBuf = ^TextBuf;
    var
      Buf : array [1..20] of Char;
      Len : Integer;
  begin
    inherited Idle;

    if COMPort.BytesInInputBuffer>0 then
    begin
      Len:=Min(COMPort.BytesInInputBuffer, SizeOf(Buf));
      COMPort.Read(Buf, Len);
      (* This should be faster than writing to TermText, but is still SLOW *)
      Term^.StrWrite(PTextBuf(@Buf)^, Len);
    end;
  end;


  (*******************************************************************
    Choose COM
  *******************************************************************)
  procedure TSerTestApp.InitCOM;
  begin
    if ExecuteDialog(MakeCOMSelectDialog, @COMDataRec)=cmOK then
    begin
      with COMDataRec do
        COMPort.InitPort(COMAdr, IRQ);

      if not COMPort.LoopBackTest then
        (* Port malfunctioning or non-existent *)
        MessageBox(^C'Loop back test failed', Nil, mfWarning+mfOKButton);

      Terminal;

      Message(@Self, evCommand, cmCOMParam, Nil);
    end;
  end;


  (*******************************************************************
    Commands
  *******************************************************************)
  procedure TSerTestApp.HandleEvent;
    (*******************************************************************
      COM Speed, Parity
    *******************************************************************)
    procedure COMParam;
      const
        COMParam : record
          Speed    : Word;
          Parity   : Word;
          DataBits : Word;
          StopBits : Word;
        end = (Speed:1);

      function MakeCOMParamDialog : PDialog;
        var
          Dlg : PDialog;
          R : TRect;
          Control : PView;

      begin
        R.Assign(15, 4, 65, 19);
        New(Dlg, Init(R, 'Serial Data'));
        Dlg^.Options := Dlg^.Options or ofCenterX or ofCenterY;

        R.Assign(4, 3, 16, 10);
        Control := New(PRadioButtons, Init(R,
          NewSItem('1200',
          NewSItem('2400',
          NewSItem('9600',
          NewSItem('19200',
          NewSItem('38400',
          NewSItem('57600',
          NewSItem('115200', Nil)))))))));
        Dlg^.Insert(Control);

          R.Assign(3, 2, 13, 3);
          Dlg^.Insert(New(PLabel, Init(R, '~B~aud rate', Control)));

        R.Assign(19, 3, 31, 7);
        Control := New(PRadioButtons, Init(R,
          NewSItem('None',
          NewSItem('Even',
          NewSItem('Mark',
          NewSItem('Space', Nil))))));
        Dlg^.Insert(Control);

          R.Assign(18, 2, 25, 3);
          Dlg^.Insert(New(PLabel, Init(R, '~P~arity', Control)));

        R.Assign(34, 3, 46, 5);
        Control := New(PRadioButtons, Init(R,
          NewSItem('8',
          NewSItem('7', Nil))));
        Dlg^.Insert(Control);

          R.Assign(33, 2, 43, 3);
          Dlg^.Insert(New(PLabel, Init(R, '~D~ata bits', Control)));

        R.Assign(34, 7, 46, 9);
        Control := New(PRadioButtons, Init(R,
          NewSItem('1',
          NewSItem('2', Nil))));
        Dlg^.Insert(Control);

          R.Assign(33, 6, 43, 7);
          Dlg^.Insert(New(PLabel, Init(R, '~S~top bits', Control)));

        R.Assign(9, 12, 19, 14);
        Control := New(PButton, Init(R, 'O~K~', cmOK, bfDefault));
        Control^.HelpCtx := hcOK;
        Dlg^.Insert(Control);

        R.Assign(21, 12, 31, 14);
        Control := New(PButton, Init(R, 'Cancel', cmCancel, bfNormal));
        Control^.HelpCtx := hcCancel;
        Dlg^.Insert(Control);

        R.Assign(33, 12, 43, 14);
        Control := New(PButton, Init(R, 'Help', cmHelp, bfGrabFocus));
        Dlg^.Insert(Control);

        Dlg^.SelectNext(False);
        MakeCOMParamDialog:= Dlg;
      end;

      const
        SpeedTbl : array [0..6] of Longint =
          (1200, 2400, 9600, 19200, 38400, 56700, 115200);
        ParityTbl : array [0..3] of ParityType =
          (None, Even, Mark, Space);

    begin
      if Application^.ExecuteDialog(MakeCOMParamDialog, @COMParam)=cmOK then
        with COMParam do
          COMPort.SetBaud(SpeedTbl[Speed],
                          ParityTbl[Parity],
                          8-DataBits,
                          1+StopBits);
    end;

    (*******************************************************************
      Send Hayes dial command
    *******************************************************************)
    procedure Dial;
      const
        Number : String = '';

      (*******************************************************************
        Simple dial number dialog
      *******************************************************************)
      function MakeDialDialog : PDialog;
        var
          Dlg : PDialog;
          R : TRect;
          Control : PView;
      begin
        R.Assign(20, 7, 60, 16);
        New(Dlg, Init(R, 'Dial a Number'));
        Dlg^.Options := Dlg^.Options or ofCentered;

        R.Assign(8, 3, 31, 4);
        Control := New(PInputLine, Init(R, 30));
        Dlg^.Insert(Control);
          PInputLine(Control)^.Validator := New(PFilterValidator, Init(['0'..'9',' ']));

          R.Assign(7, 2, 14, 3);
          Dlg^.Insert(New(PLabel, Init(R, '~N~umber', Control)));

        R.Assign(3, 6, 13, 8);
        Control := New(PButton, Init(R, 'O~K~', cmOK, bfDefault));
        Control^.HelpCtx := hcOK;
        Dlg^.Insert(Control);

        R.Assign(15, 6, 25, 8);
        Control := New(PButton, Init(R, 'Cancel', cmCancel, bfNormal));
        Control^.HelpCtx := hcCancel;
        Dlg^.Insert(Control);

        R.Assign(27, 6, 37, 8);
        Control := New(PButton, Init(R, 'Help', cmHelp, bfGrabFocus));
        Dlg^.Insert(Control);

        Dlg^.SelectNext(False);
        MakeDialDialog := Dlg;
      end;

    begin
      if (Application^.ExecuteDialog(MakeDialDialog, @Number)=cmOK) and
         (Number<>'') then
        COMPort.WriteString('ATDT'+Number+^M);
    end;

    (*******************************************************************
      Send Hayes hangup
    *******************************************************************)
    procedure Hangup;
    begin
      Notice('', ^M^M^C'Hanging up');
      Delay(3*18);                                (* 3 seconds *)
      COMPort.WriteString('+++');
      Delay(3*18);                                (* 3 seconds *)
      COMPort.WriteString('ATH0'^M);
      NoNotice;
    end;

  begin
    inherited HandleEvent(Event);

    if Event.What=evCommand then
    begin
      case Event.Command of
        cmCOMSelect:  ExecuteDialog(MakeCOMSelectDialog, @COMDataRec);
        cmCOMParam:   COMParam;
        cmDial:       Dial;
        cmHangup:     Hangup;
      end;
    end;
  end;


  (*******************************************************************
    Menu bar
  *******************************************************************)
  procedure TSerTestApp.InitMenubar;
    var
      R : TRect;
  begin
    GetExtent(R);
    R.B.Y:=R.A.Y+1;
    MenuBar:=New(PMenuBar, Init(R, NewMenu(
      NewSubMenu('~F~ile', hcNoContext, NewMenu(
        NewItem('~D~OS shell', '', kbNoKey, cmDosShell, hcDosShell,
        NewItem('E~x~it', 'Alt+X', kbAltX, cmQuit, hcExit,
        Nil))),
      NewSubMenu('~A~ctions', hcNoContext, NewMenu(
        NewItem('~D~ial', '', kbNoKey, cmDial, hcNoContext,
        NewItem('~H~angup', 'Alt+H', kbAltH, cmHangup, hcNoContext,
        Nil))),
      NewSubMenu('~O~ptions', hcNoContext, NewMenu(
        NewItem('~C~OM port...', '', kbNoKey, cmCOMSelect, hctoyOCOMSelect,
        NewItem('COM ~p~arameters...', '', kbNoKey, cmCOMParam, hcNoContext,
        Nil))),
    Nil))))));
  end;


  (*******************************************************************
    Terminal window
  *******************************************************************)
  procedure TSerTestApp.Terminal;
    var
      W : PWindow;
      R : TRect;
  begin
    Desktop^.GetExtent(R);
    New(W, Init(R, 'Terminal', 0));

    W^.GetExtent(R);
    R.Grow(-1,-1);

    New(Term, Init(R,
      W^.StandardScrollBar(sbHorizontal + sbHandleKeyboard),
      W^.StandardScrollBar(sbVertical + sbHandleKeyboard),
      TermBufSize));

    W^.Insert(Term);
    AssignDevice(TermText, Term);
    Rewrite(TermText);

    InsertWindow(W);

    Term^.COMPort:=@COMPort;
  end;


    (*******************************************************************
    *******************************************************************)

  var
    SerTestApp : TSerTestApp;

begin
  SerTestApp.Init;
  SerTestApp.Run;
  SerTestApp.Done;
end.


