// Filename: _ASK.CPP
// Used in batch files to ask user a question
// and return an errorlevel code to DOS with
// exit().  Works similar to YN.COM or
// Norton Utilities' ASK.EXE.

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <time.h>        // For time() function
#include <ctype.h>       // For toupper() function
#include <new.h>         // For new and delete

const int MAX_WAIT=9999;   // Max number of seconds to wait before default
const int MAX_ERROR_MESSAGE = 71;  // Most characters in an error message

// Function Prototypes
char *PadStr(char *, int);
char ValidResponse(char, char *);
int ErrorHandler(char *);
void OutOfMemory(void);

int main(int argc, char *argv[])  // 1st Parameter is question,
                                  // 2nd parm is valid response list,
                                  // 3rd parm is default value,
                                  // 4th parm is timeout value.
{
   set_new_handler(OutOfMemory);
   int errorlevel, counter, timeout, xpos, ypos;
   char ans[1], answer;
   char *parmlist[26][1];
   time_t t, t2;          // t is current time, t2 is timeout time
   char *cperror;         // Error string used in calls to ErrorHnadler
   char *TimeString;      // Used to display amount of seconds left
   answer = ' ' ;
   t = 1 ;
   t2 = 2;

   cperror = new char[MAX_ERROR_MESSAGE];
   TimeString = new char[5];

   if (argc < 4)
     { itoa(argc, TimeString, 10) ;
       sprintf (cperror, "%s%s%s", "Argument count (", TimeString, ") is too low.") ;
       exit(ErrorHandler(cperror)) ;
     }

   ans[1]=argv[3][8];              // Default value, format is "DEFAULT=Y"
   ans[1] = toupper(ans[1]);       // Make upper case
   *parmlist[2] = strupr(argv[2]); // Make choice list upper case

   cout << argv[1] ;               // Ask the question

   xpos = wherex();
   ypos = wherey();

   if (argv[4] != NULL)
     { char *p;
       p = new char[5];
       int i;
       p = strpbrk(argv[4], "=") + 1;  // offset 1 past "=" and point to
                                       // timeout value
       timeout = atoi(p);              // convert string to int
       if (timeout > MAX_WAIT || timeout < 0)
       { sprintf (cperror, "%s%i%s", "TIMEOUT must be between 0 and ", MAX_WAIT, ".") ;
         exit(ErrorHandler(cperror)) ;
       }

       time(&t);
       t2 = t + timeout;
       flushall();
       while ( t<t2 )
         { time(&t);                       // put curr time into t
           _setcursortype(_NOCURSOR);      // Erase Cursor
           i=1;
           if (timeout > 999) i=2;
           gotoxy(78-i, ypos);
           itoa(t2-t, TimeString, 10);
           cout << PadStr(TimeString, 2+i) ; // display seconds left
           if (kbhit())                      // if a key is hit, break
           { gotoxy(xpos + 1,ypos);
             answer = toupper(getche()) ;
             if (answer=='\r')           // If user pressed enter
             { ungetch(answer) ;
               break ;
             }
             if (ValidResponse(answer, argv[2]))  // If answer is valid
             { ans[1] = answer;                   // assign it to ans[1]
               ungetch(answer) ;
               break ;
             }
           }
         }
     }
   _setcursortype(_NORMALCURSOR);            // Restore cursor
   while (! kbhit() && (answer == ' '))     // Wait til key pressed
     { if (t >= t2) break ;
       gotoxy(xpos + 1, ypos);
       answer = toupper(getche());        // Get the response
       if (answer=='\r')           // If user pressed enter
         { ungetch(answer) ;
           break ;
         }

       if (ValidResponse(answer, argv[2]))  // If answer is valid
         { ans[1] = answer;        // assign it to ans[1]
           break ;
         }
       else
         {
           answer = ' ' ;         // To keep us in the loop
         }
     }

   if (argv[4] != NULL)
   {  gotoxy(76, ypos);
      cout << "    ";               // erase seconds left
      gotoxy(xpos + 2, ypos);
   }

   cout << "\n" ;                   // New line

   // Set errorlevel based upon where in the list the is user's response
   for (counter=0; counter<strlen(argv[2]); counter++)
     {  if (ans[1]==argv[2][counter])
          errorlevel = counter+1;
     }

   exit(errorlevel);                 // Return value to operating system.
   return 0;
}


/**************************************************************************
 * FUNCTION:     PadStr                                                   *
 * PURPOSE:      Left Pads a field item w/blanks                          *
 *                                                                        *
 * PARAMETERS:   char *pS    - pointer to the field item to be padded.    *
 *               int  nWidth - Width of field.                            *
 *                                                                        *
 * RETURN VALUE: None                                                     *
 **************************************************************************/
char *PadStr( char *pS, int nWidth )
{
  int i, iLen;
  static char szBuf[80];

  if ( ( iLen = strlen( pS ) ) == nWidth )
  {
    return ( pS );
  }

  strcpy( szBuf + nWidth - iLen, pS );

  iLen = nWidth - iLen;
  for ( i = 0; i < iLen; i++ )
  {
    *( szBuf + i ) = ' ';
  }

  return ( szBuf );
}

/**************************************************************************
 * FUNCTION:     ValidResponse                                            *
 * PURPOSE:      Checks if answer is in valid response list               *
 *                                                                        *
 * PARAMETERS:   char c      - character to be checked.                   *
 *               char *pS    - pointer to the valid resonse list.         *
 *                                                                        *
 * RETURN VALUE: Logical                                                  *
 **************************************************************************/
char ValidResponse( char c, char *pS )
{
   int counter;
   for (counter=0; counter<strlen(pS); counter++)
   {  if (c==pS[counter])
        return 1;
   }
  cout << "\a";                         // Erase invalid response and beep
  return 0;
}

/**************************************************************************
 * FUNCTION:     ErrorHandler                                             *
 * PURPOSE:      Diplays error message and returns error number           *
 *                                                                        *
 * PARAMETERS:   char *pS    - pointer to error message string            *
 *                                                                        *
 * RETURN VALUE: Integer                                                  *
 **************************************************************************/
int ErrorHandler( char *pS )
{
   cout << "\a\nError:  " << pS ;
   cout << "\n\n" ;
   cout << "Description:  _ASK.EXE asks the user a question in batch files and returns\n";
   cout << "an errorlevel code to DOS.  This program works similar to YN.COM or ASK.EXE.\n\n" ;
   cout << "Usage:  1st parameter is question to ask, 2nd parameter is valid response list,\n" ;
   cout << "3rd parameter is the default value (in case the user presses <Enter>), the\n" ;
   cout << "optional 4th parameter is a timeout value (in seconds).\n\n" ;
   cout << "Example lines in a DOS batch file:\n\n   _ASK \"Load SmartDrive?\" yn DEFAULT=y TIMEOUT=11\n" ;
   cout << "   IF ERRORLEVEL 2 GOTO NoCache\n" ;
   cout << "   IF ERRORLEVEL 1 GOTO SmartDrv\n" ;
   cout << "   :SmartDrv\n" ;
   cout << "   C:\\WINDOWS\\SMARTDRV.EXE\n" ;
   cout << "   :NoCache\n\n" ;
   cout << "1995 - Freeware from Access Data Systems, Michael P. Lascuola CIS [73301,101].\n" ;
   return 255;
}

/**************************************************************************
 * FUNCTION:     OutOfMemory                                              *
 * PURPOSE:      Calls ErrorHandler if new fails                          *
 *                                                                        *
 * PARAMETERS:   none                                                     *
 *                                                                        *
 * RETURN VALUE: none                                                     *
 **************************************************************************/
void OutOfMemory(void)
{
   exit(ErrorHandler("Cannot allocate memory.")) ;
}